@@ -22,18 +22,17 @@ A REST API for Salt
 
         .. versionadded:: 0.8.2
     debug : ``False``
-        Starts a for-development web server instead of the production-ready web
-        server.
-
-        Does not use SSL and ignores the certificate configuration options.
+        Starts the web server in development mode. It will reload itself when
+        the underlying code is changed and will output more debugging info.
     ssl_crt
-        Required when ``debug`` is ``False``
-
         The path to a SSL certificate. (See below)
     ssl_key
-        Required when ``debug`` is ``False``
-
         The path to the private key for your SSL certificate. (See below)
+    disable_ssl
+        A flag to disable SSL. Warning: your Salt authentication credentials
+        will be sent in the clear!
+
+        .. versionadded:: 0.8.3
     static
         A filesystem path to static HTML/JavaScript/CSS/image assets.
     static_path : ``/static``
@@ -47,7 +46,7 @@ A REST API for Salt
 
         .. versionadded:: 0.8.2
     app_path : ``/app``
-        The URL prefix to use for serving the HTML file specifed in the ``app``
+        The URL prefix to use for serving the HTML file specified in the ``app``
         setting. This should be a simple name containing no slashes.
 
         Any path information after the specified path is ignored; this is
@@ -64,9 +63,10 @@ A REST API for Salt
           ssl_crt: /etc/pki/tls/certs/localhost.crt
           ssl_key: /etc/pki/tls/certs/localhost.key
 
-    The REST interface requires a secure HTTPS connection. You must provide an
-    SSL certificate to use. If you don't already have a certificate and don't
-    wish to buy one, you can generate a self-signed certificate using the
+    The REST interface strongly recommends a secure HTTPS connection since Salt
+    authentication credentials will be sent over the wire. If you don't already
+    have a certificate and don't wish to buy one, you can generate a
+    self-signed certificate using the
     :py:func:`~salt.modules.tls.create_self_signed_cert` function in Salt (note
     the dependencies for this module):
 
@@ -111,7 +111,7 @@ dictionaries. This allows you to specify multiple commands in a single request.
     lowstate
         A dictionary containing various keys that instruct Salt which command
         to run, where that command lives, any parameters for that command, any
-        authentication credientials, what returner to use, etc.
+        authentication credentials, what returner to use, etc.
 
         Salt uses the lowstate data format internally in many places to pass
         command data between functions. Salt also uses lowstate for the
@@ -159,8 +159,10 @@ functionality.
 
 # Import Python libs
 import functools
+import logging
 import os
 import json
+import textwrap
 
 # Import third-party libs
 import cherrypy
@@ -169,12 +171,11 @@ import yaml
 # Import Salt libs
 import salt
 import salt.auth
-import salt.log
 
 # Import salt-api libs
 import saltapi
 
-logger = salt.log.logging.getLogger(__name__)
+logger = logging.getLogger(__name__)
 
 
 def salt_token_tool():
@@ -189,18 +190,39 @@ def salt_token_tool():
         cherrypy.request.cookie['session_id'] = x_auth
 
 
+def salt_ip_verify_tool():
+    '''
+    If there is a list of restricted IPs, verify current
+    client is coming from one of those IPs.
+    '''
+    # This is overly cumbersome and crude,
+    # But, it's also safe... ish...
+    salt_config = cherrypy.config.get('saltopts', None)
+    if salt_config:
+        cherrypy_conf = salt_config.get('rest_cherrypy', None)
+        if cherrypy_conf:
+            auth_ip_list = cherrypy_conf.get('authorized_ips', None)
+            if auth_ip_list:
+                logger.debug("Found IP list: {0}".format(auth_ip_list))
+                rem_ip = cherrypy.request.headers.get('Remote-Addr', None)
+                logger.debug("Request from IP: {0}".format(rem_ip))
+                if not rem_ip in auth_ip_list:
+                    logger.error("Blocked IP: {0}".format(rem_ip))
+                    cherrypy.response.status = 403
+                    return {
+                        'status': cherrypy.response.status,
+                        'return': "Bad IP",
+                    }
+    request = cherrypy.serving.request
+    cherrypy.response.headers['Access-Control-Allow-Origin'] = '*'
+
+
 def salt_auth_tool():
     '''
     Redirect all unauthenticated requests to the login page
     '''
-    # Short-circuit for the login page
-    ignore_urls = ('/login', '/logout', '/run')
-
-    if cherrypy.request.path_info.startswith(ignore_urls):
-        return
-
-    # Otherwise redirect to the login page if the session hasn't been authed
-    if not cherrypy.session.get('token', None):
+    # Redirect to the login page if the session hasn't been authed
+    if not cherrypy.session.has_key('token'):
         raise cherrypy.InternalRedirect('/login')
 
     # Session is authenticated; inform caches
@@ -287,7 +309,7 @@ def process_request_body(fn):
 def urlencoded_processor(entity):
     '''
     Accept x-www-form-urlencoded data (run through CherryPy's formatter)
-    and reformat it into a Low State datastructure.
+    and reformat it into a Low State data structure.
 
     Since we can't easily represent complicated data structures with
     key-value pairs, any more complicated requirements (e.g. compound
@@ -315,7 +337,7 @@ def urlencoded_processor(entity):
 @process_request_body
 def json_processor(entity):
     '''
-    Unserialize raw POST data in JSON format to a Python datastructure.
+    Unserialize raw POST data in JSON format to a Python data structure.
 
     :param entity: raw POST data
     '''
@@ -329,7 +351,7 @@ def json_processor(entity):
 @process_request_body
 def yaml_processor(entity):
     '''
-    Unserialize raw POST data in YAML format to a Python datastructure.
+    Unserialize raw POST data in YAML format to a Python data structure.
 
     :param entity: raw POST data
     '''
@@ -345,7 +367,7 @@ def hypermedia_in():
     Unserialize POST/PUT data of a specified Content-Type.
 
     The following custom processors all are intended to format Low State data
-    and will place that datastructure into the request object.
+    and will place that data structure into the request object.
 
     :raises HTTPError: if the request contains a Content-Type that we do not
         have a processor for
@@ -389,6 +411,7 @@ class LowDataAdapter(object):
 
         'tools.hypermedia_out.on': True,
         'tools.hypermedia_in.on': True,
+        'tools.salt_ip_verify.on': True,
     }
 
     def __init__(self):
@@ -397,7 +420,7 @@ class LowDataAdapter(object):
 
     def exec_lowstate(self):
         '''
-        Pull a Low State datastructure from request and execute the low-data
+        Pull a Low State data structure from request and execute the low-data
         chunks through Salt. The low-data chunks will be updated to include the
         authorization token for the current session.
         '''
@@ -721,7 +744,11 @@ class Login(LowDataAdapter):
     If the request is initiated programmatically, the request must contain a
     :mailheader:`X-Auth-Token` header with valid and active session id.
     '''
-    exposed = True
+    _cp_config = dict(LowDataAdapter._cp_config, **{
+        'tools.salt_token.on': False,
+        'tools.salt_auth.on': False,
+    })
+
 
     def __init__(self, *args, **kwargs):
         super(Login, self).__init__(*args, **kwargs)
@@ -857,6 +884,11 @@ class Login(LowDataAdapter):
 
 
 class Logout(LowDataAdapter):
+    _cp_config = dict(LowDataAdapter._cp_config, **{
+        'tools.salt_token.on': False,
+        'tools.salt_auth.on': False,
+    })
+
     def POST(self):
         '''
         Destroy the currently active session and expire the session cookie
@@ -870,6 +902,21 @@ class Logout(LowDataAdapter):
 
 
 class Run(LowDataAdapter):
+    _cp_config = dict(LowDataAdapter._cp_config, **{
+        'tools.sessions.on': False,
+        'tools.salt_token.on': False,
+        'tools.salt_auth.on': False,
+    })
+
+    def exec_lowstate(self):
+        '''
+        Override exec_lowstate to avoid pulling token from the session
+        '''
+        lowstate = cherrypy.request.lowstate
+
+        for chunk in lowstate:
+            yield self.api.run(chunk)
+
     def POST(self, **kwargs):
         '''
         Run commands bypassing the normal session handling
@@ -930,6 +977,105 @@ class Run(LowDataAdapter):
         }
 
 
+class Events(object):
+    '''
+    The event bus on the Salt master exposes a large variety of things, notably
+    when executions are started on the master and also when minions ultimately
+    return their results. This URL provides a real-time window into a running
+    Salt infrastructure.
+    '''
+    exposed = True
+
+    _cp_config = dict(LowDataAdapter._cp_config, **{
+        'response.stream': True,
+        'tools.encode.encoding': 'utf-8',
+
+        # Auth handled manually below
+        'tools.salt_auth.on': False,
+
+        'tools.hypermedia_in.on': False,
+        'tools.hypermedia_out.on': False,
+    })
+
+    def __init__(self):
+        self.opts = cherrypy.config['saltopts']
+        self.auth = salt.auth.LoadAuth(self.opts)
+
+    def GET(self, token=None):
+        '''
+        Return an HTTP stream of the Salt master event bus; this stream is
+        formatted per the Server Sent Events (SSE) spec
+
+        .. versionadded:: 0.8.3
+
+        .. http:get:: /events
+
+            **Example request**::
+
+                % curl -sS localhost:8000/events
+
+            .. code-block:: http
+
+                GET /events HTTP/1.1
+                Host: localhost:8000
+
+            **Example response**:
+
+            .. code-block:: http
+
+                HTTP/1.1 200 OK
+                Connection: keep-alive
+                Cache-Control: no-cache
+                Content-Type: text/event-stream;charset=utf-8
+
+                retry: 400
+                data: {'tag': '', 'data': {'minions': ['ms-4', 'ms-3', 'ms-2', 'ms-1', 'ms-0']}}
+
+                data: {'tag': '20130802115730568475', 'data': {'jid': '20130802115730568475', 'return': True, 'retcode': 0, 'success': True, 'cmd': '_return', 'fun': 'test.ping', 'id': 'ms-1'}}
+
+        .. note:: Caveat when using CORS
+
+            Browser clients currently lack Cross-origin resource sharing (CORS)
+            support for the ``EventSource()`` API. Cross-domain requests from a
+            browser may instead pass the :mailheader:`X-Auth-Token` value as an
+            URL parameter::
+
+                % curl -sS localhost:8000/events/6d1b722e
+
+        :status 200: success
+        :status 401: could not authenticate using provided credentials
+        '''
+        # Pulling the session token from an URL param is a workaround for
+        # browsers not supporting CORS in the EventSource API.
+        if token:
+            orig_sesion, _ = cherrypy.session.cache.get(token, ({}, None))
+            salt_token = orig_sesion.get('token')
+        else:
+            salt_token = cherrypy.session.get('token')
+
+        # Manually verify the token
+        if not salt_token or not self.auth.get_tok(salt_token):
+            raise cherrypy.InternalRedirect('/login')
+
+        # Release the session lock before starting the long-running response
+        cherrypy.session.release_lock()
+
+        cherrypy.response.headers['Content-Type'] = 'text/event-stream'
+        cherrypy.response.headers['Cache-Control'] = 'no-cache'
+        cherrypy.response.headers['Connection'] = 'keep-alive'
+
+        def listen():
+            event = salt.utils.event.SaltEvent('master', self.opts['sock_dir'])
+            stream = event.iter_events(full=True)
+
+            yield u'retry: {0}\n'.format(400)
+
+            while True:
+                yield u'data: {0}\n\n'.format(json.dumps(stream.next()))
+
+        return listen()
+
+
 class App(object):
     exposed = True
     def GET(self, *args):
@@ -948,6 +1094,7 @@ class API(object):
         'minions': Minions,
         'run': Run,
         'jobs': Jobs,
+        'events': Events,
     }
 
     def __init__(self):
@@ -979,6 +1126,9 @@ class API(object):
             },
         }
 
+        if self.apiopts.get('debug', False) == False:
+            conf['global']['environment'] = 'production'
+
         # Serve static media if the directory has been set in the configuration
         if 'static' in self.apiopts:
             conf[self.apiopts.get('static_path', '/static')] = {
@@ -1009,11 +1159,13 @@ def get_app(opts):
     # Register salt-specific hooks
     cherrypy.tools.salt_token = cherrypy.Tool('on_start_resource',
             salt_token_tool, priority=55)
+    cherrypy.tools.hypermedia_in = cherrypy.Tool('before_request_body',
+            hypermedia_in)
     cherrypy.tools.salt_auth = cherrypy.Tool('before_request_body',
             salt_auth_tool, priority=60)
     cherrypy.tools.hypermedia_out = cherrypy.Tool('before_handler',
             hypermedia_out)
-    cherrypy.tools.hypermedia_in = cherrypy.Tool('before_request_body',
-            hypermedia_in)
+    cherrypy.tools.salt_ip_verify = cherrypy.Tool('before_handler',
+            salt_ip_verify_tool)
 
     return root, apiopts, cpyopts
