@@ -39,17 +39,21 @@ An example Apache virtual host configuration::
 '''
 # pylint: disable=C0103
 
-import cherrypy
+import os
 
-from . import app
+import cherrypy
 
 def bootstrap_app():
     '''
     Grab the opts dict of the master config by trying to import Salt
     '''
-    import salt.client
-    opts = salt.client.LocalClient().opts
-    return app.get_app(opts)
+    from . import app
+    import salt.config
+
+    __opts__ = salt.config.client_config(
+            os.environ.get('SALT_MASTER_CONFIG', '/etc/salt/master'))
+    return app.get_app(__opts__)
+
 
 def get_application(*args):
     '''
@@ -61,6 +65,7 @@ def get_application(*args):
 
     def wsgi_app(environ, start_response):
         root, _, conf = opts_tuple or bootstrap_app()
+        cherrypy.config.update({'environment': 'embedded'})
 
         cherrypy.tree.mount(root, '/', conf)
         return cherrypy.tree(environ, start_response)
